@@ -27,7 +27,7 @@ import json
 import yaml
 import logging
 import logging.config
-import tornado.ioloop
+from packaging import version
 from pathlib import Path
 from orderedattrdict import AttrDict
 from gramex.config import ChainConfig, PathConfig, app_log, variables, setup_variables
@@ -127,7 +127,14 @@ def callback_commandline(commands):
 
     # If --help or -V --version is specified, print a message and end
     if kwargs.get('V') is True or kwargs.get('version') is True:
-        return console, {'msg': 'Gramex %s' % __version__}
+        pyver = '{0}.{1}.{2}'.format(*sys.version_info[:3])
+        msg = [
+            f'Gramex version: {__version__}',
+            f'Gramex path: {paths["source"]}',
+            f'Python version: {pyver}',
+            f'Python path: {sys.executable}',
+        ]
+        return console, {'msg': '\n'.join(msg)}
 
     # Any positional argument is treated as a gramex command
     if len(args) > 0:
@@ -196,12 +203,13 @@ def gramex_update(url):
     r = requests.post(url, data=json.dumps(logs))
     r.raise_for_status()
     update = r.json()
-    version = update['version']
-    if version > __version__:
-        app_log.error('Gramex %s is available. See https://learn.gramener.com/guide/', version)
-    elif version < __version__:
+    server_version = update['version']
+    if version.parse(server_version) > version.parse(__version__):
+        app_log.error('Gramex %s is available. See https://learn.gramener.com/guide/',
+                      server_version)
+    elif version.parse(server_version) < version.parse(__version__):
         app_log.warning('Gramex update: your version %s is ahead of the stable %s',
-                        __version__, version)
+                        __version__, server_version)
     else:
         app_log.debug('Gramex version %s is up to date', __version__)
     services.info.eventlog.add('update', update)
@@ -230,6 +238,13 @@ def init(force_reload=False, **kwargs):
     except Exception as e:
         app_log.exception(e)
 
+    # Add base path locations where config files are found to sys.path.
+    # This allows variables: to import files from folder where configs are defined.
+    sys.path[:] = _sys_path + [
+        str(path.absolute()) for path in paths.values()
+        if isinstance(path, Path)
+    ]
+
     # Reset variables
     variables.clear()
     variables.update(setup_variables())
@@ -292,10 +307,12 @@ def init(force_reload=False, **kwargs):
 
 def shutdown():
     '''Shut down this instance'''
-    ioloop = tornado.ioloop.IOLoop.current()
+    from . import services
+    ioloop = services.info._main_ioloop
     if ioloop_running(ioloop):
         app_log.info('Shutting down Gramex...')
-        ioloop.stop()
+        # Shut down Gramex in a thread-safe way. add_callback is the ONLY thread-safe method
+        ioloop.add_callback(ioloop.stop)
 
 
 def log(*args, **kwargs):
