@@ -8,8 +8,10 @@ import re
 import time
 import json
 import sqlalchemy as sa
+import numpy as np
 import pandas as pd
 import gramex.cache
+from packaging import version
 from tornado.escape import json_encode
 from gramex.config import merge, app_log
 from orderedattrdict import AttrDict
@@ -210,6 +212,7 @@ def filter(url, args={}, meta={}, engine=None, ext=None, columns=None,
         'limit': None,      # Limit as integer - None if not applied
         'by': [],           # Group by columns as [col, ...]
     })
+    args = dict(args)       # Do not modify the args -- keep a copy
     controls = _pop_controls(args)
     transform = _transform_fn(transform, transform_kwargs)
     url, ext, query, queryfile, kwargs = _replace(
@@ -232,8 +235,8 @@ def filter(url, args={}, meta={}, engine=None, ext=None, columns=None,
     elif engine.startswith('plugin+'):
         plugin = engine.split('+')[1]
         method = plugins[plugin]['filter']
-        data = method(url=url, controls=controls, args=args, query=query, **kwargs)
-        return _filter_frame(data, meta=meta, controls=controls, args=args)
+        return method(url=url, controls=controls, args=args, meta=meta, query=query,
+                      columns=columns, **kwargs)
     elif engine == 'sqlalchemy':
         table = kwargs.pop('table', None)
         state = kwargs.pop('state', None)
@@ -282,6 +285,7 @@ def delete(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
     if engine is None:
         engine = get_engine(url)
     meta.update({'filters': [], 'ignored': []})
+    args = dict(args)  # Do not modify the args -- keep a copy
     controls = _pop_controls(args)
     url, table, ext, query, queryfile, kwargs = _replace(
         engine, args, url, table, ext, query, queryfile, **kwargs)
@@ -327,6 +331,7 @@ def update(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
     if engine is None:
         engine = get_engine(url)
     meta.update({'filters': [], 'ignored': []})
+    args = dict(args)  # Do not modify the args -- keep a copy
     controls = _pop_controls(args)
     url, table, ext, query, queryfile, kwargs = _replace(
         engine, args, url, table, ext, query, queryfile, **kwargs)
@@ -373,6 +378,7 @@ def insert(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
     '''
     if engine is None:
         engine = get_engine(url)
+    args = dict(args)       # Do not modify the args -- keep a copy
     _pop_controls(args)
     if not args:
         raise ValueError('No args: specified')
@@ -418,9 +424,15 @@ def insert(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
             rows = _pop_columns(rows, [col.name for col in cols], meta['ignored'])
         if '.' in table:
             kwargs['schema'], table = table.rsplit('.', 1)
+        # SQLAlchemy 1.4+ only allows sa.inspect() for all table introspection
+        if version.parse(sa.__version__) >= version.parse('1.4'):
+            has_table = sa.inspect(engine).has_table(table, schema=kwargs.get('schema'))
+        # SQLAlchemy 1.3- does not have .has_table()
+        else:
+            has_table = engine.dialect.has_table(engine, table, schema=kwargs.get('schema'))
         # If the DB doesn't yet have the table, create it WITH THE PRIMARY KEYS.
-        # Note: pandas does not document engine.dialect.has_table so it might change.
-        if not engine.dialect.has_table(engine, table) and id:
+        if not has_table and id:
+            # Note: pandas does not document get_schema, so it might change.
             engine.execute(pd.io.sql.get_schema(rows, name=table, keys=id, con=engine))
 
         def insert_method(tbl, conn, keys, data_iter):
@@ -433,10 +445,14 @@ def insert(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
             sa_table = sa.Table(table, tbl.table.metadata,
                                 extend_existing=True, autoload_with=engine)
             r = conn.execute(sa_table.insert(), data)
-            # SQLAlchemy 1.4+ supports inserted_primary_key_rows, but is in beta (Nov 2020).
-            # ids = getattr(r, 'inserted_primary_key_rows', [])
-            # If we have SQLAlchemy 1.3, only single inserts have an inserted_primary_key.
-            ids = [r.inserted_primary_key] if hasattr(r, 'inserted_primary_key') else []
+            # SQLAlchemy 1.4+ supports inserted_primary_key_rows.
+            if hasattr(r, 'inserted_primary_key_rows'):
+                ids = r.inserted_primary_key_rows
+            # In SQLAlchemy 1.3, only single inserts have an inserted_primary_key.
+            elif hasattr(r, 'inserted_primary_key'):
+                ids = [r.inserted_primary_key]
+            else:
+                ids = []
             # Add non-empty IDs as a dict with associated keys
             id_cols = [col.name for col in sa_table.primary_key]
             for row in ids:
@@ -444,10 +460,22 @@ def insert(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
                     meta['inserted'].append(dict(zip(id_cols, row)))
 
         kwargs['method'] = insert_method
-        # If user passes ?col= with empty value, replace with NULL. If the column is an INT/FLOAT,
-        # type conversion int('') / float('') will fail.
-        rows.replace('', None, inplace=True)
-        pd.io.sql.to_sql(rows, table, engine, if_exists='append', index=False, **kwargs)
+        # If user passes ?col= with an empty string, replace with NULL;
+        # because, if the column is an INT/FLOAT, type conversion int('') / float('') will fail.
+        for col in rows.columns:
+            if rows[col].dtype == object:
+                rows[col].replace({'': None}, inplace=True)
+
+        # kwargs might contain additonal unexpected values, pass expected arguments explicitly
+        pd.io.sql.to_sql(
+            rows, table, engine, if_exists='append', index=False,
+            schema=kwargs.get('schema', None),
+            index_label=kwargs.get('index_label', None),
+            chunksize=kwargs.get('chunksize', None),
+            dtype=kwargs.get('dtype', None),
+            method=kwargs.get('method', None)
+        )
+
         return len(rows)
     else:
         raise ValueError('engine: %s invalid. Can be sqlalchemy|file|dataframe' % engine)
@@ -581,9 +609,25 @@ def _filter_col(col, cols):
     return None, None, None
 
 
+def _convertor(conv):
+    '''
+    Updates a type conversion function.
+
+    Booleans are converted treating '', '0', 'n', 'no', 'f', 'false' (in any case) as False.
+    Datetimes are converted using dateutil parser.
+    '''
+    # Convert based on Pandas datatype. But for boolean, convert from string as below
+    if conv in {np.bool_, bool}:
+        conv = lambda v: False if v.lower() in {'', '0', 'n', 'no', 'f', 'false'} else True # noqa
+    elif conv in {datetime}:
+        from dateutil.parser import parse
+        conv = parse
+    return conv
+
+
 def _filter_frame_col(data, key, col, op, vals, meta):
     # Apply type conversion for values
-    conv = data[col].dtype.type
+    conv = _convertor(data[col].dtype.type)
     vals = tuple(conv(val) for val in vals if val)
     if op not in {'', '!'} and len(vals) == 0:
         meta['ignored'].append((key, vals))
@@ -607,6 +651,11 @@ def _filter_frame_col(data, key, col, op, vals, meta):
     return data
 
 
+# Treat SQL type datetime as str when converting URL arguments to values.
+# Don't try to convert ?date=2022-01-01 to datetime(2022, 1, 1).
+_sql_types = {datetime: str}
+
+
 def _filter_db_col(query, method, key, col, op, vals, column, conv, meta):
     '''
     - Updates ``query`` with a method (WHERE/HAVING) that sets '<key> <op> <vals>'
@@ -614,9 +663,7 @@ def _filter_db_col(query, method, key, col, op, vals, column, conv, meta):
     - ``conv`` is a type conversion function that converts ``vals`` to the correct type
     - Updates ``meta`` with the fields used for filtering (or ignored)
     '''
-    # In PY2, .python_type returns str. We want unicode
-    sql_types = {bytes: str, pd.datetime: str}
-    conv = sql_types.get(conv, conv)
+    conv = _sql_types.get(conv, conv)
     vals = tuple(conv(val) for val in vals if val)
     if op not in {'', '!'} and len(vals) == 0:
         meta['ignored'].append((key, vals))
@@ -642,27 +689,40 @@ def _filter_db_col(query, method, key, col, op, vals, column, conv, meta):
     return query
 
 
-def _filter_sort_columns(sort_filter, cols):
+def _filter_sort_columns(controls, cols, meta):
+    '''
+    Checks ?_sort=col&_sort=-col. Returns list of (columns to sort by, ascending/not).
+
+    Updates meta['sort'] with columns to sort by,
+    and meta['ignored'] with unrecognized sort column names as ('_sort', columns)
+    '''
     sorts, ignore_sorts = [], []
-    for col in sort_filter:
-        if col in cols:
-            sorts.append((col, True))
-        elif col.startswith('-') and col[1:] in cols:
-            sorts.append((col[1:], False))
-        else:
-            ignore_sorts.append(col)
-    return sorts, ignore_sorts
+    if '_sort' in controls:
+        for col in controls['_sort']:
+            if col in cols:
+                sorts.append((col, True))
+            elif col.startswith('-') and col[1:] in cols:
+                sorts.append((col[1:], False))
+            else:
+                ignore_sorts.append(col)
+        if len(ignore_sorts) > 0:
+            meta['ignored'].append(('_sort', ignore_sorts))
+        meta['sort'] = sorts
+    return sorts
 
 
-def _filter_select_columns(col_filter, cols, meta):
+def _filter_select_columns(controls, cols, meta):
     '''
-    Checks ?_c=col&_c=-col for filter(). Takes values of ?_c= as col_filter and
-    data column names as cols. Returns 2 lists: show_cols as columns to show.
-    ignored_cols has column names not in the list, i.e. the ?_c= parameters that
-    are ignored.
+    Checks ?_c=col&_c=-col. Takes values of ?_c= as col_filter and
+    data column names as cols. Returns list of columns to show.
+
+    Updates meta['excluded'] with columns explicitly excluded,
+    and meta['ignored'] with unrecognized column names.
     '''
+    if '_c' not in controls:
+        return cols
     selected_cols, excluded_cols, ignored_cols = [], set(), []
-    for col in col_filter:
+    for col in controls['_c']:
         if col in cols:
             selected_cols.append(col)
         elif col.startswith('-') and col[1:] in cols:
@@ -673,7 +733,26 @@ def _filter_select_columns(col_filter, cols, meta):
         selected_cols = cols
     show_cols = [col for col in selected_cols if col not in excluded_cols]
     meta['excluded'] = list(excluded_cols)
-    return show_cols, ignored_cols
+    if ignored_cols:
+        meta['ignored'].append(('_c', ignored_cols))
+    return show_cols
+
+
+def _filter_offset_limit(controls, meta):
+    offset, limit = 0, None
+    if '_offset' in controls:
+        try:
+            offset = min(int(v) for v in controls['_offset'])
+        except ValueError:
+            raise ValueError('_offset not integer: %r' % controls['_offset'])
+        meta['offset'] = offset
+    if '_limit' in controls:
+        try:
+            limit = min(int(v) for v in controls['_limit'])
+        except ValueError:
+            raise ValueError('_limit not integer: %r' % controls['_limit'])
+        meta['limit'] = limit
+    return offset, limit
 
 
 def _filter_groupby_columns(by, cols, meta):
@@ -798,31 +877,17 @@ def _filter_frame(data, meta, controls, args, source='select', id=[]):
                 row = [data[col].agg(op) for col, ops in agg_dict.items() for op in ops]
                 data = pd.DataFrame([row], columns=agg_cols)
         elif '_c' in controls:
-            show_cols, hide_cols = _filter_select_columns(controls['_c'], data.columns, meta)
+            show_cols = _filter_select_columns(controls, data.columns, meta)
             data = data[show_cols]
-            if len(hide_cols) > 0:
-                meta['ignored'].append(('_c', hide_cols))
-        if '_sort' in controls:
-            meta['sort'], ignore_sorts = _filter_sort_columns(controls['_sort'], data.columns)
-            if len(meta['sort']) > 0:
-                data = data.sort_values(by=[c[0] for c in meta['sort']],
-                                        ascending=[c[1] for c in meta['sort']])
-            if len(ignore_sorts) > 0:
-                meta['ignored'].append(('_sort', ignore_sorts))
-        if '_offset' in controls:
-            try:
-                offset = min(int(v) for v in controls['_offset'])
-            except ValueError:
-                raise ValueError('_offset not integer: %r' % controls['_offset'])
+        sorts = _filter_sort_columns(controls, data.columns, meta)
+        if sorts:
+            data = data.sort_values(by=[c[0] for c in sorts],
+                                    ascending=[c[1] for c in sorts])
+        offset, limit = _filter_offset_limit(controls, meta)
+        if offset is not None:
             data = data.iloc[offset:]
-            meta['offset'] = offset
-        if '_limit' in controls:
-            try:
-                limit = min(int(v) for v in controls['_limit'])
-            except ValueError:
-                raise ValueError('_limit not integer: %r' % controls['_limit'])
+        if limit is not None:
             data = data.iloc[:limit]
-            meta['limit'] = limit
         return data
 
 
@@ -907,40 +972,30 @@ def _filter_db(engine, table, meta, controls, args, source='select', id=[]):
                     agg_cols[key] = agg_func(cols[col]).label(key)
             if not agg_cols:
                 return pd.DataFrame()
-            query = query.with_only_columns(agg_cols.values())
+            # SQLAlchemy 1.4+ only accepts positional arguments for .with_only_columns()
+            if version.parse(sa.__version__) >= version.parse('1.4'):
+                query = query.with_only_columns(*agg_cols.values())
+            # SQLAlchemy 1.3- only accepts a list for .with_only_columns()
+            else:
+                query = query.with_only_columns(agg_cols.values())
             # Apply HAVING operators
             for key, col, op, vals in cols_having:
                 query = _filter_db_col(query, query.having, key, col, op, vals,
                                        agg_cols[col], typ[col], meta)
         elif '_c' in controls:
-            show_cols, hide_cols = _filter_select_columns(controls['_c'], colslist, meta)
+            show_cols = _filter_select_columns(controls, colslist, meta)
             query = query.with_only_columns([cols[col] for col in show_cols])
-            if len(hide_cols) > 0:
-                meta['ignored'].append(('_c', hide_cols))
             if len(show_cols) == 0:
                 return pd.DataFrame()
-        if '_sort' in controls:
-            meta['sort'], ignore_sorts = _filter_sort_columns(
-                controls['_sort'], colslist + query.columns.keys())
-            for col, asc in meta['sort']:
-                orderby = sa.asc if asc else sa.desc
-                query = query.order_by(orderby(col))
-            if len(ignore_sorts) > 0:
-                meta['ignored'].append(('_sort', ignore_sorts))
-        if '_offset' in controls:
-            try:
-                offset = min(int(v) for v in controls['_offset'])
-            except ValueError:
-                raise ValueError('_offset not integer: %r' % controls['_offset'])
+        sorts = _filter_sort_columns(controls, colslist + query.columns.keys(), meta)
+        for col, asc in sorts:
+            orderby = sa.asc if asc else sa.desc
+            query = query.order_by(orderby(col))
+        offset, limit = _filter_offset_limit(controls, meta)
+        if offset is not None:
             query = query.offset(offset)
-            meta['offset'] = offset
-        if '_limit' in controls:
-            try:
-                limit = min(int(v) for v in controls['_limit'])
-            except ValueError:
-                raise ValueError('_limit not integer: %r' % controls['_limit'])
+        if limit is not None:
             query = query.limit(limit)
-            meta['limit'] = limit
         return pd.read_sql(query, engine)
 
 
@@ -1356,21 +1411,9 @@ def alter(url: str, table: str, columns: dict = None, **kwargs):
     return engine
 
 
-# NoSQL Operations
+# MongoDB Operations
 # ----------------------------------------
 
-def _type_conversion(param_list, operations=None):
-    try:
-        converted = []
-        if operations == '<' or operations == '<=':
-            converted = min(float(v) for v in param_list)
-        if operations == '>' or operations == '>=':
-            converted = max(float(v) for v in param_list)
-        return converted
-    except ValueError:
-        raise ValueError('Value is not integer: %r' % param_list)
-
-
 _mongodb_op_map = {
     '<': '$lt',
     '<~': '$lte',
@@ -1381,60 +1424,46 @@ _mongodb_op_map = {
 }
 
 
-def _filter_mongodb_col(col, op, vals, meta_cols):
-    if op in ['', '!']:
-        return {col: {_mongodb_op_map[op]: vals}}
-    elif op == '!~':
-        return {col: {"$not": {"$regex": '|'.join(vals), "$options": 'i'}}}
-    elif op == '~':
-        return {col: {"$regex": '|'.join(vals), "$options": 'i'}}
-    elif col and op in _mongodb_op_map.keys():
-        # TODO: Improve the numpy to Python type
-        convert = int if (meta_cols[col].dtype == pd.np.int64) else meta_cols[col].dtype.type
-        return {col: {_mongodb_op_map[op]: convert(val)} for val in vals}
-
-
-def _mongodb_query(args, meta_cols):
+def _mongodb_query(args, table, id=[], **kwargs):
     # Convert a query like x>=3&x>=4&x>=5 into
     # {"$or": [{x: {$gt: 3}}, {x: {$gt: 4}}, {x: $gt: 5}]}
     # TODO: ?_id= is not working
+    results = table.find().limit(1)
     conditions = []
     for key, vals in args.items():
-        col, agg, op = _filter_col(key, meta_cols)
-        if col:
-            conditions.append(_filter_mongodb_col(col, op, vals, meta_cols))
+        if len(id) and key not in id:
+            continue
+        col_names = [k for k in results[0].keys()]
+        col, agg, op = _filter_col(key, col_names)
+        add = lambda v: conditions.append({col: v})     # noqa
+        convert = _convertor(type(results[0][col])) if results else lambda v: v
+        if not col or not results:
+            continue
+        if op in {'', '!'}:
+            add({_mongodb_op_map[op]: [convert(val) for val in vals if val]})
+            if any(not val for val in vals):
+                add({'$eq' if op == '!' else '$ne': None})
+        elif op in {'>', '>~', '<', '<~'}:
+            vals = [convert(val) for val in vals if val]
+            if op == '>':
+                add({_mongodb_op_map[op]: max(vals)})
+            elif op == '<':
+                add({_mongodb_op_map[op]: min(vals)})
+            elif op == '>~':
+                add({_mongodb_op_map[op]: max(vals)})
+            elif op == '<~':
+                add({_mongodb_op_map[op]: min(vals)})
+        elif op == '!~':
+            add({"$not": {"$regex": '|'.join(vals), "$options": 'i'}})
+        elif op == '~':
+            add({"$regex": '|'.join(vals), "$options": 'i'})
+        elif col and op in _mongodb_op_map.keys():
+            add({_mongodb_op_map[op]: convert(val)} for val in vals)
+        # TODO: Handle agg
         # TODO: add meta['ignored']
     return {'$and': conditions} if len(conditions) > 1 else conditions[0] if conditions else {}
 
 
-def _controls_default(table, query=None, controls=None, meta_cols=None):
-    '''Get the controls like, _c, _sort, _offset, _limit'''
-
-    if '_c' in controls:
-        _projection = dict()
-        # _projection = {'field': 1, 'field1': -1}
-        for c in controls['_c']:
-            _projection[c] = 1
-        cursor = table.find(query, _projection)
-    else:
-        cursor = table.find(query)
-
-    if '_sort' in controls:
-        sort, ignore_sorts = _filter_sort_columns(controls['_sort'], meta_cols)
-        _sort = {key: (+1 if val else -1) for key, val in dict(sort).items()}
-
-        # sort, [('field1', 1), ('field2', -1)]
-        cursor = cursor.sort(list(_sort.items()))
-
-    if '_offset' in controls:
-        cursor = cursor.skip(int(controls['_offset'][0]))
-
-    if '_limit' in controls:
-        cursor = cursor.limit(int(controls['_limit'][0]))
-
-    return cursor
-
-
 def _mongodb_collection(url, database, collection, **kwargs):
     import pymongo
 
@@ -1457,13 +1486,29 @@ def _mongodb_json(obj):
     return result
 
 
-def _filter_mongodb(url, controls, args, database=None, collection=None, query=None, **kwargs):
+def _filter_mongodb(url, controls, args, meta, database=None, collection=None, query=None,
+                    columns=None, **kwargs):
     '''TODO: Document function and usage'''
     table = _mongodb_collection(url, database, collection, **kwargs)
-    if query is None:
-        meta_cols = pd.DataFrame(list(table.find().limit(100)))
-        query = _mongodb_query(args, meta_cols)
-    cursor = _controls_default(table, query=query, controls=controls, meta_cols=meta_cols)
+    # TODO: If data is missing, identify columns using columns:
+    cols = [k for k in table.find().limit(1)[0].keys()]
+
+    query = dict(query) if query else _mongodb_query(args, table)
+
+    show_cols = _filter_select_columns(controls, cols, meta)
+    if show_cols:
+        cursor = table.find(query, show_cols)
+    else:
+        cursor = table.find(query)
+    sorts = _filter_sort_columns(controls, cols, meta)
+    if sorts:
+        cursor = cursor.sort([(key, +1 if val else -1) for key, val in sorts])
+    offset, limit = _filter_offset_limit(controls, meta)
+    if offset is not None:
+        cursor = cursor.skip(offset)
+    if limit is not None:
+        cursor = cursor.limit(limit)
+
     data = pd.DataFrame(list(cursor))
     # Convert Object IDs into strings to allow JSON conversion
     if len(data) > 0:
@@ -1471,23 +1516,31 @@ def _filter_mongodb(url, controls, args, database=None, collection=None, query=N
         for col, val in data.iloc[0].iteritems():
             if type(val) in {bson.objectid.ObjectId}:
                 data[col] = data[col].map(str)
+
     return data
 
 
-def _delete_mongodb(url, controls, args, meta=None, database=None, collection=None, query=None,
+def _delete_mongodb(url, controls, args, meta, database=None, collection=None, query=None,
                     **kwargs):
     table = _mongodb_collection(url, database, collection, **kwargs)
-    meta_cols = pd.DataFrame(list(table.find().limit(100)))
-    query = _mongodb_query(args, meta_cols)
+    query = _mongodb_query(args, table)
     result = table.delete_many(query)
     return result.deleted_count
 
 
-def _update_mongodb(url, controls, args, meta=None, database=None, collection=None, query=None,
+def _update_mongodb(url, controls, args, meta, database=None, collection=None, query=None,
                     id=[], **kwargs):
     table = _mongodb_collection(url, database, collection, **kwargs)
-    query = _mongodb_query(args, id)
-    values = {key: val[0] for key, val in args.items()}
+    query = _mongodb_query(args, table, id=id)
+    results = list(table.find(query).limit(1))
+    if not results:
+        return 0
+
+    values = {key: val[-1] for key, val in dict(args).items() if key not in id}
+    for key in values.keys():
+        convert = _convertor(type(results[0].get(key, ''))) if results else lambda v: v
+        values[key] = convert(values[key])
+
     result = table.update_many(query, {'$set': _mongodb_json(values)})
     return result.modified_count
 
@@ -1499,6 +1552,10 @@ def _insert_mongodb(url, rows, meta=None, database=None, collection=None, **kwar
     return len(result.inserted_ids)
 
 
+# InfluxDB Operations
+# ----------------------------------------
+
+
 def _get_influxdb_schema(client, bucket):
     imports = 'import "influxdata/influxdb/schema"\n'
     meas = client.query_api().query(
@@ -1532,7 +1589,6 @@ def _influxdb_offset_limit(controls):
 
 
 def _filter_influxdb(url, controls, args, org=None, bucket=None, query=None, **kwargs):
-    args.pop("bucket")
     with _influxdb_client(url, org=org, **kwargs) as db:
         schema = _get_influxdb_schema(db, bucket)
         cols = schema["_fields"] + schema["_tags"] + schema["_measurement"]
@@ -1660,6 +1716,80 @@ def _insert_influxdb(url, rows, meta, args, bucket, **kwargs):
     return len(rows)
 
 
+# ServiceNow Operations
+# ----------------------------------------
+
+def _filter_servicenow(url, controls, args, meta, table=None, columns=None, query=None, **kwargs):
+    import pysnow
+    from gramex.config import locate
+    from urllib.parse import urlparse
+
+    urlinfo = urlparse(url)
+    c = pysnow.Client(instance=urlinfo.hostname, user=urlinfo.username, password=urlinfo.password)
+
+    config = gramex.cache.open('servicenow.yaml', 'config', rel=True)
+    if table is None:
+        table = c.resource(api_path=urlinfo.path)
+
+    # Identify column types. Take the default ServiceNow column types. Override based on user
+    table_columns = json.loads(json.dumps(config.columns[urlinfo.path]))
+    for col, colinfo in (columns or {}).items():
+        table_columns.setdefault(col, {}).update(
+            colinfo if isinstance(colinfo, dict) else {'type': colinfo})
+    for col, colinfo in table_columns.items():
+        colinfo['_convert'] = _convertor(locate(colinfo['type'], modules=['datetime']))
+    col_names = table_columns.keys()
+
+    fields = []
+    if not query:
+        offset, limit = _filter_offset_limit(controls, meta)
+        query = pysnow.QueryBuilder()
+        query_initiated = False
+
+        for key, value in args.items():
+            col, agg, op = _filter_col(key, col_names)
+            value = [table_columns[col]['_convert'](val) for val in value]
+            if query_initiated:
+                query.AND()
+            query_initiated = True
+            if op == '':
+                query.field(col).equals(value)
+            elif op == '!':
+                query.field(col).not_equals(value)
+            elif op == '>':
+                query.field(col).greater_than(min(value))
+            elif op == '>~':
+                query.field(col).greater_than_or_equal(min(value))
+            elif op == '<':
+                query.field(col).less_than(max(value))
+            elif op == '<~':
+                query.field(col).less_than_or_equal(max(value))
+            elif op == '!~':
+                query.field(col).not_contains(value[0])
+            elif op == '~':
+                query.field(col).contains(value[0])
+            else:
+                raise ValueError(f'Unknown ServiceNow operator: {op}')
+
+    offset, limit = _filter_offset_limit(controls, meta)
+    sorts = _filter_sort_columns(controls, col_names, meta)
+    for col, asc in sorts:
+        if query_initiated:
+            query.AND()
+        query_initiated = True
+        if asc:
+            query.field(col).order_ascending()
+        else:
+            query.field(col).order_descending()
+    fields.extend(_filter_select_columns(controls, col_names, meta))
+
+    if not len(query._query):
+        query = {}
+
+    response = table.get(query=query, limit=limit, offset=offset, fields=fields)
+    return pd.DataFrame(response.all())
+
+
 # add test case for inserting nested value ?parent.={child:value}
 #   curl --globoff -I -X POST 'http://127.0.0.1:9988/?x.={"2":3}&y.={"true":true}&Name=abcd'
 # add test case for updating nested value ?parent.child.={key:value}
@@ -1677,3 +1807,6 @@ plugins["influxdb"] = {
     "insert": _insert_influxdb,
     "update": _insert_influxdb,
 }
+plugins["servicenow"] = {
+    "filter": _filter_servicenow
+}
