@@ -88,9 +88,11 @@ class BaseMixin(object):
     def setup_default_kwargs(cls):
         '''
         Use default config from handlers.<Class>.* and handlers.BaseHandler.
-        Called by gramex.services.url().
+        Called directly by gramex.services.url().
+        NOTE: This updates the kwargs for setup() -- so it must be called BEFORE setup()
+        and can't be merged into setup() or called from setup().
         '''
-        c = cls.conf.setdefault('kwargs', {})
+        c = cls.conf.kwargs
         merge(c, objectpath(conf, 'handlers.' + cls.conf.handler, {}), mode='setdefault')
         merge(c, objectpath(conf, 'handlers.BaseHandler', {}), mode='setdefault')
 
@@ -431,7 +433,12 @@ class BaseMixin(object):
 
     def debug_exception(self, typ, value, tb):
         super(BaseHandler, self).log_exception(typ, value, tb)
-        import ipdb as pdb              # noqa
+        try:
+            import ipdb as pdb  # noqa: T100
+        except ImportError:
+            import pdb  # noqa: T100
+            import warnings
+            warnings.warn('"pip install ipdb" for better debugging')
         pdb.post_mortem(tb)
 
     def _write_custom_error(self, status_code, **kwargs):
@@ -589,6 +596,7 @@ class BaseMixin(object):
                 raise HTTPError(BAD_REQUEST, reason=reason)
             self._session_store.dump('otp:' + otp, None)
             self.session['user'] = otp_data['user']
+        # ToDo: apikey = headers.get('X-Gramex-OTP') or self.get_argument('gramex-otp', None)
 
     def set_last_visited(self):
         '''
@@ -890,7 +898,7 @@ class BaseWebSocketHandler(WebSocketHandler, BaseMixin):
     def get(self, *args, **kwargs):
         for method in self._on_init_methods:
             method(self)
-        super(BaseWebSocketHandler, self).get(*args, **kwargs)
+        yield super(BaseWebSocketHandler, self).get(*args, **kwargs)
 
     def on_close(self):
         # Loop through class-level callbacks
