@@ -17,7 +17,11 @@ class ComicHandler(BaseHandler):
             _info['node'] = Node(port=9967, cwd=os.path.join(variables['GRAMEXAPPS'], 'ui'))
         node = _info['node']
         # Take the last argument, i.e. ?name=dee&name=ava => {name: ava}
-        args = {key: vals[-1] for key, vals in self.args.items()}
+
+        if len(self.args.keys()):
+            args = {key: vals[-1] for key, vals in self.args.items()}
+        else:
+            args = {"name": "aryan", "emotion": "angry", "pose": "handsinpocket"}
         # Fetch the results via Comicgen
         result = yield node.js(code='return require("comicgen")(require("fs"))(args)', args=args)
         # If there's no error, set the header and send the result
