@@ -136,8 +136,8 @@ class MLHandler(FormHandler):
         return df
 
     @classmethod
-    def store_data(cls, df, append=False):
-        df.to_hdf(cls.data_store, format="table", key="data", append=append)
+    def store_data(cls, df, append=False, **kwargs):
+        df.to_hdf(cls.data_store, format="table", key="data", append=append, **kwargs)
         return cls.load_data(df)
 
     @classmethod
@@ -300,6 +300,7 @@ class MLHandler(FormHandler):
 
     @coroutine
     def prepare(self):
+        super(MLHandler, self).prepare()
         flattened = {}
         for k, v in self.args.items():
             if not isinstance(TRANSFORMS.get(k), list) and isinstance(v, list) and len(v) == 1:
@@ -309,6 +310,7 @@ class MLHandler(FormHandler):
 
     @coroutine
     def get(self, *path_args, **path_kwargs):
+        self.set_header('Content-Type', 'application/json')
         if '_params' in self.args:
             params = {
                 'opts': self.config_store.load('transform'),
@@ -335,7 +337,6 @@ class MLHandler(FormHandler):
                     self.write(fout.read())
             elif '_model' in self.args:
                 self.write(json.dumps(self.get_opt('params'), indent=2))
-
             else:
                 try:
                     data_args = {k: v for k, v in self.args.items() if not k.startswith('_')}
@@ -347,7 +348,6 @@ class MLHandler(FormHandler):
                     app_log.debug(err.msg)
                     data = []
                 if len(data) > 0:
-                    self.set_header('Content-Type', 'application/json')
                     data = data.drop([self.get_opt('target_col')], axis=1, errors='ignore')
                     prediction = yield gramex.service.threadpool.submit(
                         self._predict, data)
@@ -396,6 +396,7 @@ class MLHandler(FormHandler):
         if action not in ACTIONS:
             raise HTTPError(BAD_REQUEST, f'Action {action} not supported.')
         res = yield gramex.service.threadpool.submit(getattr(self, f"_{action}"))
+        self.set_header('Content-Type', 'application/json')
         self.write(json.dumps(res, indent=2, cls=CustomJSONEncoder))
         super(MLHandler, self).post(*path_args, **path_kwargs)
 
@@ -428,7 +429,7 @@ class MLHandler(FormHandler):
         self.config_store.purge()
 
     def _delete_cache(self):
-        self.store_data(pd.DataFrame())
+        self.store_data(pd.DataFrame(), mode="w")
 
     def _delete_opts(self):
         for opt in set(self.get_arguments('_opts')) & TRANSFORMS.keys():
