@@ -33,7 +33,8 @@ class WebSocketHandler(BaseWebSocketHandler):
                 setattr(cls, method, build_transform(
                     kwargs[method],
                     vars=OrderedDict((arg, None) for arg in override_methods[method]),
-                    filename='url:%s.%s' % (cls.name, method)))
+                    filename='url:%s.%s' % (cls.name, method),
+                    iter=False))
 
     @classmethod
     def setup(cls, **kwargs):
