@@ -3,7 +3,6 @@ Defines command line services to install, setup and run apps.
 '''
 import io
 import os
-import re
 import sys
 import time
 import yaml
@@ -13,15 +12,14 @@ import string
 import shutil
 import datetime
 import requests
-from glob import glob
 from shutilwhich import which
 from pathlib import Path
-from subprocess import Popen, check_output, CalledProcessError      # nosec
+# subprocess is safe since it runs developer-initiated commands
+from subprocess import Popen, check_output, CalledProcessError      # nosec: developer-initiated
 from orderedattrdict import AttrDict
 from orderedattrdict.yamlutils import AttrDictYAMLLoader
 from zipfile import ZipFile
 from tornado.template import Template
-from orderedattrdict.yamlutils import from_yaml         # noqa
 import gramex
 import gramex.license
 from gramex.config import ChainConfig, PathConfig, variables, app_log, slug
@@ -45,7 +43,7 @@ install: |
     replaced by the target directory.
 
     After installation, runs "gramex setup" which runs the Makefile, setup.ps1,
-    setup.sh, requirements.txt, setup.py, yarn/npm install and bower install.
+    setup.sh, requirements.txt, setup.py, bower install, npm install, yarn install.
 
     Installed apps:
     {apps}
@@ -64,8 +62,9 @@ setup: |
         - bash setup.sh
         - pip install --upgrade -r requirements.txt
         - python setup.py
-        - yarn/npm install
-        - bower install
+        - bower --allow-root install
+        - npm install
+        - yarn install --prefer-offline
 
 run: |
     usage: gramex run <app> [--target=DIR] [--dir=DIR] [--<options>=<value>]
@@ -125,12 +124,18 @@ service: |
         gramex service stop
 
 init: |
-    usage: gramex init [--target=DIR]
+    gramex init [--target=DIR]
+    gramex init minimal [--target=DIR]
 
     Initializes a Gramex project at the current or target dir. Specifically, it:
     - Sets up a git repo
-    - Install supporting files for a gramex project
-    - Runs gramex setup (which runs yarn/npm install and other dependencies)
+    - Install supporting files for a Gramex project from a template
+      - "gramex init" sets up dependencies for a local system
+      - "gramex init minimal" sets up minimal dependencies
+    - Runs gramex setup (which runs npm install and other dependencies)
+
+    Options:
+      --target <path>               # Location to install at. Defaults to
 
 mail: |
     gramex mail <key>               # Send mail named <key>
@@ -149,7 +154,8 @@ license: |
     gramex license accept           # Accept Gramex license
     gramex license reject           # Reject Gramex license
 '''
-usage = yaml.load(usage, Loader=AttrDictYAMLLoader)     # nosec
+# yaml.load is safe since it only reads the string above, not user-created content
+usage = yaml.load(usage, Loader=AttrDictYAMLLoader)     # nosec: frozen input
 
 
 class TryAgainError(Exception):
@@ -165,7 +171,7 @@ except NameError:
         raise exc_info[1]
 else:
     # On Windows systems, try harder
-    def _ensure_remove(function, path, exc_info):
+    def _ensure_remove(func, path, exc_info):
         '''onerror callback for rmtree that tries hard to delete files'''
         if issubclass(exc_info[0], WindowsError):
             import winerror
@@ -187,10 +193,12 @@ else:
                     except WindowsError:
                         pass
             # npm creates windows shortcuts that shutil.rmtree cannot delete.
-            # os.listdir failes with a PATH_NOT_FOUND. Delete these and try again
-            elif function == os.listdir and exc_info[1].winerror == winerror.ERROR_PATH_NOT_FOUND:
+            # os.listdir/scandir fails with a PATH_NOT_FOUND.
+            # Delete these using win32com and try again.
+            elif (exc_info[1].winerror == winerror.ERROR_PATH_NOT_FOUND and
+                  func in {os.listdir, os.scandir}):
                 app_log.error('Cannot delete %s', path)
-                from win32com.shell import shell, shellcon
+                from win32com.shell import shell, shellcon  # type:ignore
                 options = shellcon.FOF_NOCONFIRMATION | shellcon.FOF_NOERRORUI
                 code, err = shell.SHFileOperation((0, shellcon.FO_DELETE, path, None, options))
                 if code == 0:
@@ -314,10 +322,11 @@ def run_command(config):
     target = config.target
     cygcheck, cygpath, kwargs = which('cygcheck'), which('cygpath'), {'universal_newlines': True}
     if cygcheck is not None and cygpath is not None:
+        # subprocess.check_output is safe here since these are developer-initiated
         app_path = check_output([cygpath, '-au', which(appcmd[0])], **kwargs).strip()   # nosec
         is_cygwin_app = check_output([cygcheck, '-f', app_path], **kwargs).strip()      # nosec
         if is_cygwin_app:
-            target = check_output([cygpath, '-au', target], **kwargs).strip()           # n osec
+            target = check_output([cygpath, '-au', target], **kwargs).strip()           # nosec
     # Replace TARGET with the actual target
     if 'TARGET' in appcmd:
         appcmd = [target if arg == 'TARGET' else arg for arg in appcmd]
@@ -327,35 +336,26 @@ def run_command(config):
     if not safe_rmtree(config.target):
         app_log.error('Cannot delete target %s. Aborting installation', config.target)
         return
-    proc = Popen(appcmd, bufsize=-1, **kwargs)      # nosec
+    proc = Popen(appcmd, bufsize=-1, **kwargs)      # nosec: developer-initiated
     proc.communicate()
     return proc.returncode
 
 
-# Setup file configurations.
-# Structure: {File: {exe: cmd}}
-# If File exists, then if exe exists, run cmd.
-# For example, if package.json exists:
-#   then if yarn exists, run yarn install
-#   else if npm exists, run npm install
-setup_paths = '''
-Makefile:
-    make: '"{EXE}"'
-setup.ps1:
-    powershell: '"{EXE}" -File "{FILE}"'
-setup.sh:
-    bash: '"{EXE}" "{FILE}"'
-requirements.txt:
-    pip: '"{EXE}" install -r "{FILE}"'
-setup.py:
-    python: '"{EXE}" "{FILE}"'
-package.json:
-    yarn: '"{EXE}" install --prefer-offline'
-    npm: '"{EXE}" install'
-bower.json:
-    bower: '"{EXE}" --allow-root install'
-'''
-setup_paths = yaml.load(setup_paths, Loader=AttrDictYAMLLoader)     # nosec
+# Setup file configurations. If {file} exists, then if {exe} exists, run {cmd}.
+setup_paths = [
+    [{'file': 'Makefile', 'exe': 'make', 'cmd': '"{exe}"'}],
+    [{'file': 'setup.ps1', 'exe': 'powershell', 'cmd': '"{exe}" -File "{file}"'}],
+    [{'file': 'setup.sh', 'exe': 'bash', 'cmd': '"{exe}" "{file}"'}],
+    [{'file': 'requirements.txt', 'exe': 'pip', 'cmd': '"{exe}" install -r "{file}"'}],
+    [{'file': 'setup.py', 'exe': 'python', 'cmd': '"{exe}" "{file}"'}],
+    [{'file': 'bower.json', 'exe': 'bower', 'cmd': '"{exe}" --allow-root install'}],
+    [
+        {'file': 'package-lock.json', 'exe': 'npm', 'cmd': '"{exe}" ci'},
+        {'file': 'yarn.lock', 'exe': 'yarn', 'cmd': '"{exe}" install --prefer-offline'},
+        {'file': 'package.json', 'exe': 'npm', 'cmd': '"{exe}" install'},
+        {'file': 'package.json', 'exe': 'yarn', 'cmd': '"{exe}" install --prefer-offline'},
+    ]
+]
 
 
 def run_setup(target):
@@ -367,16 +367,6 @@ def run_setup(target):
     - A relative path to the Gramex apps/ folder
 
     Returns the absolute path of the final target path.
-
-    This supports:
-
-    - ``make`` (if Makefile exists)
-    - ``powershell -File setup.ps1``
-    - ``bash setup.sh``
-    - ``pip install -r requirements.txt``
-    - ``python setup.py``
-    - ``yarn install`` else ``npm install``
-    - ``bower --allow-root install``
     '''
     if not os.path.exists(target):
         app_target = os.path.join(variables['GRAMEXPATH'], 'apps', target)
@@ -385,19 +375,21 @@ def run_setup(target):
         target = app_target
     target = os.path.abspath(target)
     app_log.info('Setting up %s', target)
-    for file, runners in setup_paths.items():
-        setup_file = os.path.join(target, file)
-        if not os.path.exists(setup_file):
-            continue
-        for exe, cmd in runners.items():
-            exe_path = which(exe)
-            if exe_path is not None:
-                cmd = cmd.format(FILE=setup_file, EXE=exe_path)
+    for configs in setup_paths:
+        config_match, ran_cmd = None, False
+        for config in configs:
+            setup_file = os.path.join(target, config['file'])
+            exe_path = which(config['exe'])
+            if os.path.exists(setup_file):
+                config_match = config
+            if config_match and exe_path is not None:
+                cmd = config['cmd'].format(file=setup_file, exe=exe_path)
                 app_log.info('Running %s', cmd)
                 _run_console(cmd, cwd=target)
+                ran_cmd = True
                 break
-        else:
-            app_log.warning('Skipping %s. No %s found', setup_file, exe)
+        if config_match and not ran_cmd:
+            app_log.warning('Skipping %s. No %s found', config_match['file'], config_match['exe'])
 
 
 app_dir = Path(variables.get('GRAMEXDATA')) / 'apps'
@@ -583,7 +575,7 @@ def service(args, kwargs):
 def _check_output(cmd, default=b'', **kwargs):
     '''Run cmd and return output. Return default in case the command fails'''
     try:
-        return check_output(shlex.split(cmd), **kwargs).strip()     # nosec
+        return check_output(shlex.split(cmd), **kwargs).strip()     # nosec: developer-initiated
     # OSError is raised if the cmd is not found.
     # CalledProcessError is raised if the cmd returns an error.
     except (OSError, CalledProcessError):
@@ -607,36 +599,17 @@ def _mkdir(path):
         os.makedirs(path)
 
 
-def _copy(source, target, template_data=None):
-    '''
-    Copy single directory or file (as binary) from source to target.
-    Warn if target exists, or source is not file/directory, and exit.
-    If template_data is specified, treat source as a Tornado template.
-    '''
-    if os.path.exists(target):
-        app_log.warning('Skip existing %s', target)
-    elif os.path.isdir(source):
-        _mkdir(target)
-    elif os.path.isfile(source):
-        app_log.info('Copy file %s', source)
-        with io.open(source, 'rb') as handle:
-            result = handle.read()
-            if template_data is not None:
-                from mimetypes import guess_type
-                filetype = guess_type(source)[0] or 'text/unknown'
-                if re.match('text/.*|application/(json|javascript)', filetype):
-                    result = Template(result).generate(**template_data)
-        with io.open(target, 'wb') as handle:
-            handle.write(result)
-    else:
-        app_log.warning('Skip unknown file %s', source)
-
-
 def init(args, kwargs):
     '''Create Gramex scaffolding files.'''
-    if len(args) > 1:
+    if len(args) > 2:
         app_log.error(show_usage('init'))
         return
+    if len(args) == 0:
+        args.append('default')
+    source_dir = os.path.join(variables['GRAMEXPATH'], 'apps', 'init', args[0])
+    if not os.path.exists(source_dir):
+        app_log.error(f'Unknown init template {args[0]}')
+
     kwargs.setdefault('target', os.getcwd())
     app_log.info('Initializing Gramex project at %s', kwargs.target)
     data = {
@@ -666,22 +639,30 @@ def init(args, kwargs):
         except OSError:
             data['git_lfs'] = None
 
-    # Copy all directories & files (as templates)
-    source_dir = os.path.join(variables['GRAMEXPATH'], 'apps', 'init')
+    # Copy all directories & files. Files with '.template.' are treated as templates.
     for root, dirs, files in os.walk(source_dir):
+        relpath = os.path.relpath(root, start=source_dir)
         for name in dirs + files:
             source = os.path.join(root, name)
-            relpath = os.path.relpath(root, start=source_dir)
-            target = os.path.join(kwargs.target, relpath, name.replace('appname', appname))
-            _copy(source, target, template_data=data)
-    for empty_dir in ('img', 'data'):
-        _mkdir(os.path.join(kwargs.target, 'assets', empty_dir))
-    # Copy error files as-is (not as templates)
-    error_dir = os.path.join(kwargs.target, 'error')
-    _mkdir(error_dir)
-    for source in glob(os.path.join(variables['GRAMEXPATH'], 'handlers', '?0?.html')):
-        target = os.path.join(error_dir, os.path.basename(source))
-        _copy(source, target)
+            targetname = name.replace('$appname', appname)
+            template_data = None
+            if '.template.' in name:
+                targetname, template_data = name.replace('.template.', '.'), data
+            target = os.path.join(kwargs.target, relpath, targetname)
+            if os.path.exists(target):
+                app_log.warning('Skip existing %s', target)
+            elif os.path.isdir(source):
+                _mkdir(target)
+            elif os.path.isfile(source):
+                app_log.info('Copy file %s', source)
+                with io.open(source, 'rb') as handle:
+                    result = handle.read()
+                    if template_data is not None:
+                        result = Template(result).generate(**template_data)
+                with io.open(target, 'wb') as handle:
+                    handle.write(result)
+            else:
+                app_log.warning('Skip unknown file %s', source)
 
     run_setup(kwargs.target)
 
