@@ -15,9 +15,8 @@ from collections import OrderedDict
 from orderedattrdict import AttrDict
 from tornado.template import Template
 from orderedattrdict.yamlutils import AttrDictYAMLLoader
-from pandas.util.testing import assert_frame_equal as afe
 from nose.tools import eq_, ok_, assert_raises
-from . import folder, tests_dir, remove_if_possible, dbutils
+from . import folder, tests_dir, remove_if_possible, dbutils, afe
 
 cache_folder = os.path.join(folder, 'test_cache')
 state_file = os.path.join(cache_folder, '.state')
@@ -203,7 +202,7 @@ class TestOpen(unittest.TestCase):
         afe(gramex.cache.open(path, sheet_name='table', table='CensusTable'),
             gramex.cache.open(path, sheet_name='table', range='$D1:F$23'))
         # Load named range. The "sales" named range is the same as the sheet "sales"
-        afe(gramex.cache.open(path, sheet_name='sales', defined_name='sales'),
+        afe(gramex.cache.open(path, sheet_name='sales', name='sales'),
             gramex.cache.open(path, sheet_name='sales'))
         # Test failure conditions, edge cases, etc.
 
