@@ -7,7 +7,7 @@ import requests
 import lxml.html
 import pandas as pd
 import sqlalchemy as sa
-from nose.tools import eq_, ok_
+from nose.tools import eq_, ok_, assert_not_equal as neq_
 from nose.plugins.skip import SkipTest
 from tornado.web import create_signed_value
 from urllib.parse import urlencode, urljoin
@@ -188,10 +188,12 @@ class LoginMixin(object):
 class LoginFailureMixin(object):
     def check_delay(self, start, min=None, max=None):
         t = time.time()
+        # Give a 0.1s buffer in case of timing delays
+        buffer = 0.1
         if min is not None and min > 0:
-            self.assertGreaterEqual(t - start, min)
+            self.assertGreaterEqual(t - start, min - buffer)
         if max is not None and max > 0:
-            self.assertLessEqual(t - start, max)
+            self.assertLessEqual(t - start, max + buffer)
         return t
 
     def test_slow_down_attacks(self, retries=3):
@@ -483,6 +485,71 @@ class TestLookup(AuthBase):
         eq_(session['user']['gender'], 'female')
 
 
+class TestRules(AuthBase):
+    url = server.base_url + '/auth/rulesdb'
+
+    def test_default(self):
+        self.login_ok('alpha', 'alpha', check_next='/')
+        session = self.session.get(server.base_url + '/auth/session').json()
+        eq_(session['user']['gender'], 'male')
+
+        self.login_ok('beta', 'beta', check_next='/')
+        session = self.session.get(server.base_url + '/auth/session').json()
+        eq_(session['user']['gender'], 'female')
+        eq_(session['user']['team'], 'Gramex')
+
+        # When selector is a nonexistent attribute, nothing changes
+        self.login_ok('γ', 'gamma', check_next='/')
+        session = self.session.get(server.base_url + '/auth/session').json()
+        eq_(session['user']['gender'], 'female')
+
+        # Check if the empty string match for Gamma works
+        eq_(session['user']['email'], 'gamma@null.com')
+
+        upass = {'γ': 'gamma'}
+        for user in 'alpha beta γ'.split():
+            self.login_ok(user, upass.get(user, user), check_next='/')
+            session = self.session.get(server.base_url + '/auth/session').json()
+            # Check that no users have attributes defined in an unreachable rule
+            neq_(session['user']['email'], 'none@none.com')
+            # Check that everyone is team: Gramex
+            eq_(session['user']['team'], 'Gramex')
+
+
+class TestRulesFile(TestRules):
+    url = server.base_url + '/auth/rulesfile'
+
+    def test_default(self):
+        super(TestRulesFile, self).test_default()
+        upass = {'γ': 'gamma'}
+        expires_duration = 3.14
+        for user in 'alpha beta γ'.split():
+            self.login_ok(user, upass.get(user, user), check_next='/')
+            session = self.session.get(server.base_url + '/auth/session').json()
+            eq_(session['user']['cookie_expires'], expires_duration)
+            eq_(session['user']['expiry'], '2050-12-31T00:00:00+05:30')
+
+
+class TestNoRules(AuthBase):
+    url = server.base_url + '/auth/norules'
+
+    def test_default(self):
+        self.login_ok('alpha', 'alpha', check_next='/')
+        session = self.session.get(server.base_url + '/auth/session').json()
+        eq_(session['user']['gender'], 'female')
+
+        self.login_ok('beta', 'beta', check_next='/')
+        session = self.session.get(server.base_url + '/auth/session').json()
+        eq_(session['user']['gender'], 'male')
+        eq_(session['user']['team'], 'ग्रामेक्स')
+
+        # When selector is a nonexistent attribute, nothing changes
+        self.login_ok('γ', 'gamma', check_next='/')
+        session = self.session.get(server.base_url + '/auth/session').json()
+        eq_(session['user']['gender'], 'male')
+        eq_(session['user']['empty'], '')
+
+
 class DBAuthBase(AuthBase):
     @staticmethod
     def create_database(url, table):
