@@ -318,3 +318,14 @@ class TestCaptureHandlerChrome(TestCaptureHandler):
         # Text elements are grouped differently in PDFs generated by Window and Linux
         ok_('Header' in layout[0] or 'Header' in layout[1])
         ok_(re.search(r'1\s*/\s*1', layout[-1]))
+
+    def test_browser_disconnect(self, delay=0.2):
+        # Disconnect the browser after the response is received
+        content = self.capture.jpg(url=server.base_url + self.url, _test_disconnect=1)
+        self.check_img(content)
+        # Re-connecting to the instance takes a bit of time. This may vary based on system speed
+        # If this test fails due to ReadTimeout exception, increase the delay.
+        time.sleep(delay)
+        # Capture the same image again. The server should re-connect automatically
+        content = self.capture.jpg(url=server.base_url + self.url)
+        self.check_img(content)
