@@ -16,9 +16,10 @@ from sklearn.datasets import make_circles as sk_make_circles
 from tornado import gen
 from tornado.web import RequestHandler, MissingArgumentError, HTTPError
 from tornado.gen import coroutine
+from tornado.concurrent import Future
 from tornado.httpclient import AsyncHTTPClient
 from concurrent.futures import ThreadPoolExecutor
-from gramex.cache import Subprocess
+from gramex.cache import Subprocess, CustomJSONEncoder
 from gramex.services import info
 from gramex.services.emailer import SMTPStub
 from gramex.handlers import BaseHandler
@@ -76,14 +77,18 @@ def str_callback(value, callback):
 
 def iterator_async(handler):
     for val in handler.get_arguments('x'):
-        future = gen.Task(str_callback, val)
+        future = Future()
+        str_callback(val, callback=future.set_result)
         yield future
 
 
 @gen.coroutine
 def async_args(*args, **kwargs):
     '''Run params_as_json asynchronously'''
-    result = yield gen.Task(params_as_json, *args, **kwargs)
+    future = Future()
+    kwargs['callback'] = future.set_result
+    params_as_json(*args, **kwargs)
+    result = yield future
     raise gen.Return(result)
 
 
@@ -116,7 +121,7 @@ def async_calc(handler):
     rows = 1000
     cols = ['A', 'B', 'C']
     df = pd.DataFrame(
-        pd.np.arange(rows * len(cols)).reshape((rows, len(cols))),
+        np.arange(rows * len(cols)).reshape((rows, len(cols))),
         columns=cols)
     df = df % 4
     counts = yield [thread_pool.submit(count_group, df, col) for col in cols]
@@ -197,7 +202,7 @@ def session(handler):
     var = handler.get_argument('var', None)
     if var is not None:
         handler.session['var'] = var
-    return json.dumps(handler.session, indent=4)
+    return json.dumps(handler.session, indent=4, cls=CustomJSONEncoder)
 
 
 def encrypt(handler, content):
@@ -402,7 +407,7 @@ def numpytypes(handler):
         'uint8', 'uint16', 'uint32', 'uint64',
         'float16', 'float32', 'float64',
         'bool_', 'object_', 'string_', 'unicode_'}
-    result = {t: getattr(pd.np, t)(1) for t in supported_types}
+    result = {t: getattr(np, t)(1) for t in supported_types}
     return result
 
 
@@ -471,12 +476,12 @@ def get_state_info():
 def make_circles():
     X, y = sk_make_circles(noise=0.05, factor=0.4)  # NOQA: N806
     out = os.path.join(os.path.dirname(__file__), 'circles.csv')
-    pd.DataFrame(pd.np.c_[X, y], columns=['X1', 'X2', 'y']).to_csv(
+    pd.DataFrame(np.c_[X, y], columns=['X1', 'X2', 'y']).to_csv(
         out, encoding='utf-8', index=False)
 
 
 def transform_circles(df, *argss, **kwargs):
-    df[['X1', 'X2']] = pd.np.exp(-df[['X1', 'X2']].values ** 2)
+    df[['X1', 'X2']] = np.exp(-df[['X1', 'X2']].values ** 2)
     return df
 
 
@@ -505,7 +510,7 @@ def test_function(
         i1: Annotated[int, 'First value'] = 0,  # noqa
         i2: Annotated[int, 'Second value'] = 0,  # noqa
         s1: str = 'Total',
-        n1: np.int = 0,
+        n1: int = 0,
         n2: np.int64 = 0,
         h: Header = '',
         code: int = OK):
